//------------------------------------------------------------
#include "config.h"
#include "device.h"
//------------------------------------------------------------
#include <QApplication>
#include <QFile>
#include <QTextStream>
#include <QProcess>
#include <iostream>
//------------------------------------------------------------
using namespace std;
//------------------------------------------------------------
Config::Config()
{
	settings = new QSettings("TrollWare", "QMounter");
}
//------------------------------------------------------------
void Config::create()
{
	cout << "Creating configuration file..." << endl;

	settings->setValue("FIRST_RUN",  0);
	settings->setValue("HIDE_AT_START",  true);
	settings->setValue("ACTIONS",  QStringList()
		<< "MOUNTED|Umount|umount %p"
		<< "UMOUNTED|Mount|mount %p"
		<< "UMOUNTED|Fatsort|sudo fatsort %d"
	);
	settings->setValue("BLACK_LIST_OF_DEVICES",  QStringList()
		<< "none"
		<< "binfmt_misc"
	);

	cout << "Configuration file created." << endl;
}
//------------------------------------------------------------
void Config::read()
{
	if ( settings->value("FIRST_RUN", 1).toInt() )
		create();
	actions = settings->value("ACTIONS").toStringList();
	hideAtStart = settings->value("HIDE_AT_START").toBool();
	bl_of_devices = settings->value("BLACK_LIST_OF_DEVICES").toStringList();
	windowGeometry = settings->value("WINDOW_GEOMETRY").toRect();
	windowSize = settings->value("WINDOW_SIZE").toSize();
}
//------------------------------------------------------------
void Config::write()
{
	settings->setValue("ACTIONS",  actions);
	settings->setValue("HIDE_AT_START",  hideAtStart);
	settings->setValue("BLACK_LIST_OF_DEVICES",  bl_of_devices);
	settings->setValue("WINDOW_SIZE", windowSize);
	settings->setValue("WINDOW_GEOMETRY", windowGeometry);
}
//------------------------------------------------------------
void Config::setDevices()
{
	Device::devices.clear();

	QFile file("/proc/partitions");
	if (!file.open(QIODevice::ReadOnly | QIODevice::Text))
	{
		cerr << endl << "Can not open the file: /proc/partitions" << endl;
		qApp->quit();
	}
	
	QTextStream in(&file);
	QString line;
	while (!(line = in.readLine()).isNull())
	{
		if ((new QRegExp("^\\s+\\d"))->indexIn(line) == -1)
			continue;
		
		if ((line.section(*(new QRegExp("\\s+")), 2, 2).toInt() % 16) == 0)
			continue;
			
		Device *tmpDevice = new Device(line.section(*(new QRegExp("\\s+")), 4, 4));
		Device::devices.append(tmpDevice);
	}
	file.close();
}
//------------------------------------------------------------
