//------------------------------------------------------------
#include <iostream>
#include <QMessageBox>
#include <QTextCodec>
//------------------------------------------------------------
#include "first_f.h"
#include "device.h"
//------------------------------------------------------------
using namespace std;
//------------------------------------------------------------
first_f::first_f( QWidget * parent, Qt::WFlags f) 
	: QMainWindow(parent, f)
{
	setupUi(this);
	
	config = new Config;
	config->read();
	
	if (!config->windowSize.isNull())
		resize(config->windowSize);
		
	if (!config->windowGeometry.isNull())
		setGeometry(config->windowGeometry);

	setActions();
	on_actionUpdate_triggered();
	
	sysTray = new QSystemTrayIcon(QIcon(":/images/computer.png"), this);
	sysTray->show();
	
	if (!config->hideAtStart)
		show();
	
	QObject::connect(sysTray, SIGNAL(activated(QSystemTrayIcon::ActivationReason)), this, SLOT(sysTray_activated(QSystemTrayIcon::ActivationReason)));
}
//------------------------------------------------------------
void first_f::setActions()
{
	for (int i = 0; i < config->actions.count(); i++)
	{
		Action *tmpAction = new Action( config->actions[i], menuActions );
		Action::actions.append(tmpAction);
		QObject::connect(tmpAction->menuItem, SIGNAL(triggered()), this, SLOT(actionTriggered()));
		QObject::connect(tmpAction->process, SIGNAL(finished(int, QProcess::ExitStatus)), this, SLOT(commandFinished(int, QProcess::ExitStatus)));
		QObject::connect(tmpAction->process, SIGNAL(error(QProcess::ProcessError)), this, SLOT(commandError(QProcess::ProcessError)));
	}
}
//------------------------------------------------------------
void first_f::updateDevices()
{
	devices_l->clear();
	for (int i = 0; i < Device::devices.count(); i++)
	{
		if (config->bl_of_devices.contains(Device::devices[i]->file))
			continue;
		devices_l->addTopLevelItem( new QTreeWidgetItem( QStringList()
			<< Device::devices[i]->file
			<< Device::devices[i]->point
			<< Device::devices[i]->file_system
			<< (Device::devices[i]->free_size + "/" + Device::devices[i]->full_size)
		));
	}
	
	for (int i = 0; i < Action::actions.count(); i++)
		Action::actions[i]->menuItem->setEnabled(false);
}
//------------------------------------------------------------
void first_f::actionTriggered()
{
	this->setEnabled(false);
	Action::actionOfTitle( ((QAction*)sender())->text() )->execute(Device::deviceOfFile(devices_l->currentItem()->text(0)));
}
//------------------------------------------------------------
void first_f::on_devices_l_itemSelectionChanged()
{
	if (!devices_l->currentItem())
		return;

	for (int i = 0; i < Action::actions.count(); i++)
	{
		if (Device::deviceOfFile(devices_l->currentItem()->text(0))->mounted == (Action::actions[i]->type == 0))
			Action::actions[i]->menuItem->setEnabled(true);
		else
			Action::actions[i]->menuItem->setEnabled(false);
	}
}
//------------------------------------------------------------
void first_f::on_actionUpdate_triggered()
{
	config->setDevices();
	updateDevices();
}
//------------------------------------------------------------
void first_f::commandFinished(int exitCode, QProcess::ExitStatus exitStatus)
{
	cout << endl << "Exit code: " << exitCode << ", Exit status: " << exitStatus << endl;
	if (exitCode)
	{
		QByteArray tmpEcoded = ((QProcess*)sender())->readAllStandardError();
		QTextCodec *codec = QTextCodec::codecForLocale();
		QString tmpNornal = codec->toUnicode(tmpEcoded);
 
		cout << tmpNornal.toAscii().data() << endl;
		QMessageBox::critical(this, "Execute error", tmpNornal);
	}
	this->setEnabled(true);
	on_actionUpdate_triggered();
}
//------------------------------------------------------------
void first_f::commandError(QProcess::ProcessError error)
{
	QMessageBox::critical(this, "Execute error", "Command did not execute.");
	this->setEnabled(true);
}
//------------------------------------------------------------
void first_f::on_actionAbout_Qt_triggered()
{
	QMessageBox::aboutQt(this);
}
//------------------------------------------------------------
void first_f::on_actionAbout_triggered()
{
	QMessageBox::about(this, "About..."," \
		<h1 align=\"center\">QMounter</h1> \
		<br> \
		<table cellpadding=\"5\"> \
			<tr><td>Version:</td><td align=\"center\">0.1.1</td></tr> \
			<tr><td>Homepage:</td><td align=\"center\"><a href=\"http://qt-apps.org/content/show.php?content=61054\">http://qt-apps.org/</a></td></tr> \
			<tr><td>Autors:</td><td align=\"center\">DubHead</td> \
		</table> \
	");
}
//------------------------------------------------------------
void first_f::closeEvent(QCloseEvent *event)
{
	config->windowSize = size();
	config->windowGeometry = geometry();
	config->write();
}
//------------------------------------------------------------
void first_f::sysTray_activated(QSystemTrayIcon::ActivationReason reason)
{
	if (reason != QSystemTrayIcon::Trigger)
		return;
		
	if (isVisible())
	{
		config->windowGeometry = geometry();
		hide();
	}
	else
	{
		setGeometry(config->windowGeometry);
		show();
	}
}
//------------------------------------------------------------
